/*===============================*\
This version shows how making
the routines, sprite-specific,
you can make the code a lot simpler
\*===============================*/

#include <libps.h>
#include "MyPad.h"

#define OT_LENGTH		2
#define PACKETMAX		2048
#define PACKETMAX2		(PACKETMAX*24)
#define SCREEN_WIDTH	320
#define SCREEN_HEIGHT	256

#define SPRITE_ADDRESS 0x80090000

GsOT		WorldOT[2];	
GsOT_TAG	OTTags[2][1<<OT_LENGTH];
static PACKET packetArea[2][PACKETMAX2];
volatile u_char *bb0, *bb1;


void main(void);
void InitGame(void);
static u_long ReadPad(void);
void ProperInitialiseTexture(long TIMdata, GsIMAGE *imageInfo);


void main(void) {
	int activeBuff;
	u_long pad;
	GsSPRITE spriteHandler;
	GsIMAGE imageHandler;
	RECT rect;
	
	InitGame();

	GsGetTimInfo((u_long *)(SPRITE_ADDRESS+4), &imageHandler);

	// Same as rect.x = ... etc
	// width is 32 instead of 64 as image is 8-bit
	setRECT(&rect, 320, 0, 32, 32);
	LoadImage(&rect, imageHandler.pixel);
	DrawSync(0);

	setRECT(&rect, 320, 32, 256, 1);
	LoadImage(&rect, imageHandler.clut);
	DrawSync(0);
	
	spriteHandler.x = 160;
	spriteHandler.y = 128;
	spriteHandler.w = 64;
	spriteHandler.attribute = 1<<24;
	spriteHandler.h = 32;
	spriteHandler.u = 0;
	spriteHandler.v = 0;
	spriteHandler.cx = 320;
	spriteHandler.cy = 32;
	spriteHandler.r = 128;
	spriteHandler.g = 128;
	spriteHandler.b = 128;
	spriteHandler.mx = 32;
	spriteHandler.my = 16;
	spriteHandler.scalex = ONE;
	spriteHandler.scaley = ONE;
	spriteHandler.rotate = 0;
	spriteHandler.tpage = GetTPage(1, 0, 320, 0);
	
	do {
		pad = ReadPad();
		activeBuff = GsGetActiveBuff();
		GsSetWorkBase((PACKET *)packetArea[activeBuff]);
		GsClearOt(0, 0, &WorldOT[activeBuff]);
		
		
		// Draw Here
		GsSortSprite(&spriteHandler, &WorldOT[activeBuff], 0);
		
		DrawSync(0);
		VSync(0);
		GsSwapDispBuff();
		
		GsSortClear(0, 0, 0, &WorldOT[activeBuff]);
		GsDrawOt(&WorldOT[activeBuff]);
	} while ((pad != PADLstart+PADLselect) && (pad != PADRselect+PADRstart));
}


void InitGame(void) {
	int i;
	
	SetVideoMode(MODE_PAL);
	GsInitGraph(SCREEN_WIDTH, SCREEN_HEIGHT, GsINTER|GsOFSGPU, 1, 0);
	
	GsDefDispBuff(0, 0, 0, SCREEN_HEIGHT);
	GsDISPENV.screen.h = 255;
	
	GetPadBuf(&bb0, &bb1);
	
	for (i = 0; i < 2; i++) {
		WorldOT[i].length = OT_LENGTH;
		WorldOT[i].org = OTTags[i];
		GsClearOt(0,0,&WorldOT[i]);
	}
}


u_long ReadPad(void) {
	return(~(*(bb0+3) | *(bb0+2) << 8 | *(bb1+3) << 16 | *(bb1+2) << 24));
}
